package tab.list

import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.Check
import androidx.compose.material.icons.outlined.Close
import androidx.compose.material.icons.outlined.Info
import androidx.compose.material.icons.outlined.Warning
import androidx.compose.material3.Icon
import androidx.compose.material3.ListItem
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import cafe.adriel.voyager.core.screen.Screen
import data.EGroup
import data.ENumber

class ENumberList(private val eNumbers: List<EGroup>, private val searchText: MutableState<String>) : Screen {

    @OptIn(ExperimentalFoundationApi::class)
    @Composable
    override fun Content() {
        val displayItems = searchFilter(query = searchText.value, eGroups = eNumbers)

        println(displayItems)

        if (displayItems.isEmpty()) {
            Column (
                modifier = Modifier
                    .fillMaxWidth(),
                horizontalAlignment = Alignment.CenterHorizontally,
            ) {
                Text(text = "Sorry, we couldn't find any results :(")
            }
            return
        }

        LazyColumn(contentPadding = PaddingValues(6.dp)) {
            displayItems.forEach { eGroup ->

                if (displayItems.isNotEmpty()) {
                    stickyHeader {
                        Header(group = eGroup.group)
                    }
                }

                items(eGroup.numbers) { eNumber ->
                    ItemList(eNumber = eNumber)
                }
            }
        }
    }

    @Composable
    private fun Header(group: Int) {
        val text = if (group < 10) {
            "E${group}00 - E${group}99"
        } else {
            "E1000 - E1599"
        }

        Text(
            text = text,
            modifier = Modifier
                .fillMaxWidth()
                .background(MaterialTheme.colorScheme.surface)
                .padding(8.dp)
        )
    }

    @Composable
    private fun ItemList(eNumber: ENumber) {
        ListItem(
            headlineContent = { Text(text = eNumber.name) },
            supportingContent = { Text(text = eNumber.number) },
            leadingContent = {
                VeganStatusIcon(
                    vegan = eNumber.vegan,
                    notVegan = eNumber.notVegan
                )
            },
        )
    }

    @Composable
    private fun VeganStatusIcon(vegan: Boolean, notVegan: Boolean) {
        val (icon, status) = if (vegan and !notVegan) {
            Pair(Icons.Outlined.Check, "Vegan")
        } else if (vegan and notVegan) {
            Pair(Icons.Outlined.Warning, "Potentially Not Vegan")
        } else if (!vegan and notVegan) {
            Pair(Icons.Outlined.Close, "Not Vegan")
        } else {
            Pair(Icons.Outlined.Info, "Unknown")
        }

        Icon(
            imageVector = icon,
            contentDescription = status
        )
    }

    private fun searchFilter(query: String, eGroups: List<EGroup>): List<EGroup> {
        if (query.isEmpty()) {
            return eGroups
        }

        val filtered = eGroups.map { eGroup ->
            eGroup.copy(numbers = eGroup.numbers.filter { eNumber ->
                eNumber.number.contains(query, ignoreCase = true)
            })
        }.filter { it.numbers.isNotEmpty() }

        return filtered
    }
}
